import bluej.extensions.*;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;
import javax.swing.SwingConstants;
import javax.swing.BorderFactory;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

/**
 * Clase GetterSetterFrameObject.
 * 
 * @author (Milton Jes&uacute;s Vera Contreras - miltonjesusvc@ufps.edu.co) 
 * @version 0.0000000000000001 --> Math.sin(Math.PI-Double.MIN_VALUE) --> :)
 */
public class GetterSetterFrameObject extends JFrame implements ActionListener
{
   /**Arreglo de cuadros de texto donde de editaran las propiedades*/
   private GetterSetterTextField [] fields;
   
   private JButton okButton;

   private JButton exitButton;
   
   private JButton aboutButton;
   
   private JButton nextButton;
   private JButton previousButton;
   private JButton firstButton;
   private JButton lastButton;
   private JComboBox elementCombo;
   
   private ArrayList controls;
   
   private GetterSetterIntrospector gsi;

   public GetterSetterFrameObject(BObject theBObject) throws Exception
   {
      gsi = new GetterSetterIntrospector(theBObject);
      initComponents();
      registerListener();
      makeVisible();
      new ProgressInvoker(this, new InvokerGetter(this));
   }//end constructor
   
   /**Make visible*/
   public void makeVisible()
   {
     this.pack();
     this.setSize(500,this.getHeight());
     this.setLocation((int)(Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2 - this.getWidth()/2),(int)(Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2 - this.getHeight()/2));
     this.setResizable(false);
     this.setAlwaysOnTop(true);
     this.setVisible(true);
   }//end makeVisible
   
   /**Generate the GUI*/
   protected void initComponents()
   {
     JPanel frame = new JPanel();
     JLabel title = new JLabel();
     this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
     this.setLayout(new BorderLayout());
     frame.setLayout(new BorderLayout());     
     frame.add(generateDataPanel(), BorderLayout.CENTER);
     frame.add(generateButtonPanel(), BorderLayout.SOUTH);
     frame.setBorder(BorderFactory.createTitledBorder(""));
     title.setBorder(BorderFactory.createTitledBorder(""));
     try{title.setText("Editing " + gsi.getTheBObject().getInstanceName() + " With GetterSetter");}catch(Exception e){}
     title.setHorizontalAlignment(SwingConstants.CENTER);
     this.getContentPane().add(frame, BorderLayout.CENTER);
     this.getContentPane().add(title, BorderLayout.NORTH);     
     try{this.setTitle("Editing " + gsi.getTheBObject().getInstanceName() + " With GetterSetter");}catch(Exception e){}
     if(gsi.theBObjectIsArray()) this.getContentPane().add(generatePanelArray(), BorderLayout.SOUTH);
   }//end initComponents
   
   /**Generate the data JPanel*/
   private JPanel generateDataPanel()
   {
    String [] propertiesWithGetSet = gsi.getPropertiesWithGetSet();
    JPanel dataPanel = new JPanel();
    JLabel label;
    GetterSetterTextField text;
    dataPanel.setLayout(new GridLayout(propertiesWithGetSet.length,2));
    fields = new GetterSetterTextField [propertiesWithGetSet.length];
    for(int i=0; i<propertiesWithGetSet.length;i++){
      label = new JLabel(propertiesWithGetSet[i]+":");
      label.setHorizontalAlignment(JLabel.CENTER);
      text = new GetterSetterTextField (gsi, propertiesWithGetSet[i]);
      dataPanel.add(label);
      dataPanel.add(text);
      text.setEditable(true);
      fields[i] = text;
    }//end for generate data JTextField and JLabel
    return dataPanel;
   }//end generateDataPanel
   
   private void registerListener()
   {
      this.controls = new ArrayList();
      this.controls.add(okButton);
      this.controls.add(exitButton);
      this.controls.add(aboutButton);
      okButton.addActionListener(this);
      exitButton.addActionListener(this);
      aboutButton.addActionListener(this);
      if(gsi.theBObjectIsArray()){
       this.controls.add(firstButton);
       this.controls.add(previousButton);
       this.controls.add(nextButton);
       this.controls.add(lastButton);
       this.controls.add(elementCombo);
       nextButton.addActionListener(this);
       previousButton.addActionListener(this);
       firstButton.addActionListener(this);
       lastButton.addActionListener(this);
       elementCombo.addActionListener(this);
      }//end isArtray
   }//end registerListenet

   /**Generate the button JPanel*/
   private JPanel generateButtonPanel()
   {
     JPanel buttonsPanel = new JPanel();
     buttonsPanel.setLayout(new FlowLayout());
     okButton = new JButton("Accept");
     exitButton = new JButton("Exit");
     aboutButton = new JButton("Help");
     buttonsPanel.add(okButton);
     buttonsPanel.add(exitButton);
     buttonsPanel.add(aboutButton);
     return buttonsPanel;
   }//end generateButtonPanel
   
   /**Generate Array Panel*/
   private JPanel generatePanelArray()
   {
     JPanel buttonsArray = new JPanel();
     buttonsArray.setLayout(new GridLayout(1,5));
     nextButton = new JButton("Next");
     previousButton = new JButton("Previous");
     firstButton = new JButton("First");
     lastButton = new JButton("Last");
     elementCombo = new JComboBox();
     buttonsArray.add(firstButton);
     buttonsArray.add(previousButton);
     buttonsArray.add(elementCombo);
     buttonsArray.add(nextButton);
     buttonsArray.add(lastButton);
     for(int i=0;i<gsi.getTotalObjects();i++)
      elementCombo.addItem(String.valueOf(i));
     return buttonsArray;
   }//end generatePanelArray
   
   /**Click event*/
   public void actionPerformed(ActionEvent evt)
   {
    int source = controls.indexOf(evt.getSource());
    switch(source){
      case 0: {
         new ProgressInvoker(this, new InvokerSetter(this));
         break;
      }//end case 0
      case 1: {
         this.dispose();
         break;
      }//end case 1
      case 2: {
         showAbout();
         break;
      }//end case 2
      case 3: {
         gsi.firstBObject();
         break;
      }//end case 3
      case 4: {
         gsi.previousBObject();
         break;
      }//end case 4
      case 5: {
         gsi.nextBObject();
         break;
      }//end case 5
      case 6: {
         gsi.lastBObject();
         break;
      }//end case 6
      case 7: {
         gsi.setTheBObjectPosition(elementCombo.getSelectedIndex());
         new ProgressInvoker(this, new InvokerGetter(this));
         break;
      }//end case 7
      default: {
      }//end default
    }//end switch
    if(source >= 3 && source <=6) elementCombo.setSelectedIndex(gsi.getTheBObjectPosition());
   }//end actionPerformed
   
   private void showAbout()
   {
             String text = "Getter Setter Extension\n";
             text      +=  "Milton Jesús Vera Contreras\n";
             text      +=  "Universidad Francisco de Paula Santander\n";
             text      +=  "Cúcuta-Colombia\n";
             text      +=  "miltonjesusvc@ufps.edu.co - miljeveco@gmail.com\n";
             text      +=  "Versión 0.0000000000000001 --> Math.sin(Math.PI-Double.MIN_VALUE) --> :)\n";
             text      +=  "Diciembre de 2018";
        JOptionPane.showMessageDialog(this, text);       
   }//end showAbout
   
   public GetterSetterTextField [] getFields(){return this.fields;}
   
}//fin class GetterSetterFrameObject
